<?php

// this is used to (re)start the script without changing map, useful for live script tweaking
$SKIP_START = false;
global $argv;
if (isset($argv[1]) && $argv[1] == "skip") {
	$SKIP_START = true;
}

include_once("internal/helpers.php");
include_once("internal/comms.php");
include_once("internal/log.php");

include_once("internal/error_setup.php");


// --------------------------------------------

abstract class MetaGame {
	public $comms = NULL;
	protected $trackers = array();
	protected $time_tracker;

	// --------------------------------------------
	public function init() {
		// initialize communications to/from server
		$this->comms = new Comms();
		$this->comms->init();
		
		global $argv;
		if (test_parameter($argv, "single_player")) {
		} else {
			// now start server, if not started yet
			$this->start_server();
		}

		$this->time_tracker = microtime(true);
	}

	// --------------------------------------------
	public function run() {
		while (true) {
			$this->update();

			_log("waiting for data...");

			// blocks here until message received
			$data = $this->comms->receive();
			if ($data === false) break;

			if (!empty($data)) { 
				if (count($this->trackers) > 0) {
					$doc = new DOMDocument();
					$doc->loadXML($data);

					if ($doc->firstChild->localName == "user_quit_event") {
						_log("user quit signal detected, stopping script");
						break;
					}

					// inform all trackers about the event
					foreach ($this->trackers as $key => $tracker) {
						$tracker->handle_event($doc);
					}
				} else {
					_log("no event tracker registered, skipping message");
				}
			}
		}
	}

	// --------------------------------------------
	public function uninit() {
		_log("couldn't read from pipe, rwr server no longer running?");

		$this->comms->uninit();
	}

	// --------------------------------------------
	public function begin_match() {
		_log("begin match");
		$this->clear_trackers();
	}

	// --------------------------------------------
	protected function start_server() {
		// get_start_server_command defined in server_settings.php
		$command = get_start_server_command();
		if ($command != "") {
			$this->comms->send($command);
		} else {
			_log("not starting server");
		}
	}

	// --------------------------------------------
	public function add_tracker($tracker) {
		$this->trackers[] = $tracker;
	}

	// --------------------------------------------
	protected function clear_trackers() {
		$this->trackers = array();
	}

	// --------------------------------------------
	protected function tracker_completed($tracker) {
		// by default, nothing to do here
	}

	// --------------------------------------------
	protected function update() {
		$time_start = $this->time_tracker;
		$this->time_tracker = microtime(true); 
		$time_end = $this->time_tracker; 

		$t = $time_end - $time_start;

		_log("update: time since last time " . $t);
		_log("  trackers: " . count($this->trackers));

		// maintain trackers
		foreach ($this->trackers as $key => $tracker) {
			// track passed time here and feed it to tracker, trackers can then
			// use if for updating timers and looking out for expiration to trigger stuff
			//

			// windows + PHP + named pipes (+ the fact that PHP thread support is less than perfect) 
			// has one issue:
			// - we need to pulse dummy events from the server to keep the script not blocking at
			//   checking if there's anything to read in the input pipe
			// - the pulses come in at max 1 second intervals, so the script won't update itself faster
			//   than that except when other events happen to come in

			$tracker->update($t);

			if ($tracker->has_ended()) {
				$this->tracker_completed($tracker);
				_log("tracker " . $key . " ended, removing from list");
				// cleanup
				unset($this->trackers[$key]);
			} else if (!$tracker->has_started()) {
				// not ended and not started
				// try to start it
				$tracker->start();
			}
		}
	}

}

?>
